function [Jhat ] = test_stat(ehat0, X0)
%% Calculate the test statistic Jhat 
%% Input:
%%         ehat0: N x T 
%%            X0: N x T x p
%%             N: cross section dimension
%%             T: time dimension 
%% Output:
%%          Jhat: test statistic 
%% Last modified on 2012/8/10 

%% Step 1: Preparation

N=size(X0,1);
T=size(X0,2);
p=size(X0,3);


ehat = reshape(ehat0,N,T);                                  % N x T

X    = zeros(p,T,N);

for ii = 1:p

    X(ii,:,:) = X0(:,:,ii)';                  % p x T x N
end



ccheck=0;
if ccheck==1
M0 = eye(T) - 1/T*ones(T,T);  
for iiii= 1:N
    Xiiii = X(:,:,iiii)';                                         % T x p
    Omegai_hat= Xiiii'*M0*Xiiii/T;
    checkproblems(iiii,:)=min(eig(Omegai_hat))<10^-4;
end



X(:,:,find(checkproblems==1))=[];
ehat(find(checkproblems==1),:)=[];

N=N-sum(checkproblems);

end


%% Step 2: Calculate the test statistic    
LM0 = zeros(N,1);
B0  = zeros(N,1);
V0  = zeros(N,1);
%PFhat = Fhat*Fhat'/T;                                       % same as Fhat* pinv(Fhat'*Fhat)*Fhat'
%MFhat = eye(T) - PFhat;
M0 = eye(T) - 1/T*ones(T,T);                                %%% new added

for i= 1:N
    %% 1) Raw statistic
    Xi = X(:,:,i)';                                         % T x p
    Omegai_hat= Xi'*M0*Xi/T;                                %%% p x p modified
    PXi = M0*Xi*pinv(Omegai_hat)*Xi'*M0/T;                  %%% T x T modified
    LM0(i,1)= ehat(i,:)*PXi*ehat(i,:)';                     % scalar

    %% 2) Bias estimation
    Hihat = M0*PXi*M0;            %LU Changes                    % T x T
    Hihat_diag = diag(Hihat);                               % T x 1
    B0(i,1) = (ehat(i,:).^2)* Hihat_diag;                   % scalar

    %% 3) Variance estimation
%       tt2=((Xi'*Fhat)/T)* Fhat';                               % p x T
%       bi_hat=(Xi-tt')*Omegai_hat^(-1/2);                      % T x p

    bi_hat= M0*Xi*Omegai_hat^(-1/2); %LU changes                     % T x p
    b_epsilon_i = bi_hat.*kron(ehat(i,:)', ones(1,p));      % T x p
    b_epsilon_i_cumsum = cumsum(b_epsilon_i);               % T x p
    temp= b_epsilon_i(2:T,:).* b_epsilon_i_cumsum(1:T-1,:); % (T-1) x p
    V0(i,1) = sum( (sum(temp,2)).^2 ); 
end
LMNT = sum(LM0);                                            % scalar
BNT  = sum(B0)/sqrt(N);
VNT  = 4*sum(V0)/(N*(T^2));

% 4) Final test statistic
Jhat = ( N^(-1/2)* LMNT - BNT )/sqrt(VNT);
 %[ N^(-1/2)*LMNT  BNT  sqrt(VNT) Jhat ]


